@@ -2,19 +2,19 @@
 pub struct Rule {
     name: String,
     command: String,
-    description: String,
+    description: Option<String>,
 }
 
 impl Rule {
     pub fn new(
         name: impl Into<String>,
         command: impl Into<String>,
-        description: impl Into<String>,
+        description: Option<String>,
     ) -> Self {
         Self {
             name: name.into(),
             command: command.into(),
-            description: description.into(),
+            description,
         }
     }
 
@@ -26,7 +26,7 @@ impl Rule {
         &self.command
     }
 
-    pub fn description(&self) -> &str {
-        &self.description
+    pub fn description(&self) -> Option<&str> {
+        self.description.as_deref()
     }
 }
