@@ -9,6 +9,7 @@ use crate::{
     ir::{Build, Configuration, Rule},
     parse::parse_dynamic,
     utilities::read_file,
+    validation::BuildGraph,
 };
 use async_recursion::async_recursion;
 use futures::future::{join_all, try_join_all, FutureExt, Shared};
@@ -21,7 +22,13 @@ use std::{
     sync::Arc,
     time::SystemTime,
 };
-use tokio::{fs::metadata, io::AsyncWriteExt, process::Command, spawn, sync::Semaphore};
+use tokio::{
+    fs::metadata,
+    io::{self, AsyncWriteExt},
+    process::Command,
+    spawn,
+    sync::Semaphore,
+};
 
 type RawBuildFuture = Pin<Box<dyn Future<Output = Result<(), InfrastructureError>> + Send>>;
 type BuildFuture = Shared<RawBuildFuture>;
@@ -30,11 +37,15 @@ pub async fn run(
     configuration: Configuration,
     build_directory: &Path,
     job_limit: Option<usize>,
+    debug: bool,
 ) -> Result<(), InfrastructureError> {
+    let graph = BuildGraph::new(configuration.outputs())?;
     let context = Arc::new(Context::new(
         configuration,
+        graph,
         BuildDatabase::new(build_directory)?,
         Semaphore::new(job_limit.unwrap_or_else(num_cpus::get)),
+        debug,
     ));
 
     for output in context.configuration().default_outputs() {
@@ -51,7 +62,7 @@ pub async fn run(
 
     // Do not inline this to avoid borrowing a lock of builds.
     let futures = context
-        .builds()
+        .build_futures()
         .read()
         .await
         .values()
@@ -70,7 +81,7 @@ async fn create_build_future(
     build: &Arc<Build>,
 ) -> Result<(), InfrastructureError> {
     // Exclusive lock for atomic addition of a build job.
-    let mut builds = context.builds().write().await;
+    let mut builds = context.build_futures().write().await;
 
     if builds.contains_key(build.id()) {
         return Ok(());
@@ -95,7 +106,7 @@ async fn spawn_build_future(
                 if let Some(build) = context.configuration().outputs().get(input) {
                     create_build_future(&context, build).await?;
 
-                    context.builds().read().await[build.id()].clone()
+                    context.build_futures().read().await[build.id()].clone()
                 } else {
                     // TODO Consider registering this future as a build job of the input.
                     let input = input.to_string();
@@ -110,12 +121,16 @@ async fn spawn_build_future(
 
         // TODO Consider caching dynamic modules.
         let dynamic_configuration = if let Some(dynamic_module) = build.dynamic_module() {
-            Some(compile_dynamic(&parse_dynamic(
-                &read_file(&dynamic_module).await?,
-            )?)?)
+            let configuration =
+                compile_dynamic(&parse_dynamic(&read_file(&dynamic_module).await?)?)?;
+
+            context.build_graph().lock().await.insert(&configuration)?;
+
+            Some(configuration)
         } else {
             None
         };
+
         let dynamic_inputs = if let Some(configuration) = &dynamic_configuration {
             build
                 .outputs()
@@ -134,7 +149,7 @@ async fn spawn_build_future(
 
             create_build_future(&context, build).await?;
 
-            futures.push(context.builds().read().await[build.id()].clone());
+            futures.push(context.build_futures().read().await[build.id()].clone());
         }
 
         join_builds(futures).await?;
@@ -225,9 +240,12 @@ async fn run_rule(context: &Context, rule: &Rule) -> Result<(), InfrastructureEr
 
     let mut console = context.console().lock().await;
 
+    if context.debug() {
+        writeln(console.stderr(), rule.command()).await?;
+    }
+
     if let Some(description) = rule.description() {
-        console.stderr().write_all(description.as_bytes()).await?;
-        console.stderr().write_all("\n".as_bytes()).await?;
+        writeln(console.stderr(), description).await?;
     }
 
     console.stdout().write_all(&output.stdout).await?;
@@ -242,3 +260,13 @@ async fn run_rule(context: &Context, rule: &Rule) -> Result<(), InfrastructureEr
 
     Ok(())
 }
+
+async fn writeln(
+    writer: &mut (impl AsyncWriteExt + Unpin),
+    message: impl AsRef<str>,
+) -> Result<(), io::Error> {
+    writer.write_all(message.as_ref().as_bytes()).await?;
+    writer.write_all("\n".as_bytes()).await?;
+
+    Ok(())
+}
